class Track(): 

	def __init__(self,
		seed,
		id,
        added_at,
        added_by,
        artist_name,
        release_date,
        album_name,
        popularity,
        track_number,
        track_name,
        disc_number,
        album_id,
        genres,
        artist_id,
		acousticness,
        analysis_url,
        danceability,
        duration_ms,
        energy,
        instrumentalness,
        key,
        liveness,
        loudness,
        mode,
        speechiness,
        tempo,
        time_signature,
        track_href,
        type,
        uri,
        valence
		):
		self.seed = seed
		self.id = id
		self.added_at = added_at
		self.added_by = added_by
		self.artist_name = artist_name
		self.release_date = release_date
		self.album_name = album_name
		self.popularity = popularity
		self.track_number = track_number
		self.track_name = track_name
		self.disc_number = disc_number
		self.album_id = album_id
		self.album_genres = genres
		self.artist_id = artist_id
		self.acousticness = acousticness
		self.analysis_url = analysis_url
		self.danceability = danceability
		self.duration_ms = duration_ms
		self.energy = energy
		self.instrumentalness = instrumentalness
		self.key = key
		self.liveness = liveness
		self.loudness = loudness
		self.mode = mode
		self.speechiness = speechiness
		self.tempo = tempo
		self.time_signature = time_signature
		self.track_href = track_href
		self.type = type
		self.uri = uri
		self.valence = valence



import pandas as pd


def dataframing(
	object_track_list,
	object_track_recommendations_list,
	original_artists_list, 
	recommended_artists_list,
	playlist,
	):

###
###
###   TRACKS OUTPUT DATAFRAMES 
###
###


	tracks_dataframe = []

	for track in object_track_list:
		data = track.__dict__

		original_track_frame = pd.DataFrame({
		"seed": data["seed"],
		"id": data["id"],
		"added_at": data["added_at"],
		"added_by": data["added_by"],
		"artist_name": data["artist_name"],
		"release_date": data["release_date"],
		"album_name": data["album_name"],		
		"popularity": data["popularity"],
		"track_number": data["track_number"],		
		"track_name": data["track_name"],
		"disc_number": data["disc_number"],
		"album_id": data["album_id"],
		"album_genres": data["album_genres"],
		"artist_id": data["artist_id"],
		"acousticness": data["acousticness"],
	    "danceability": data["danceability"],
	    "duration_ms": data["duration_ms"],
	    "energy": data["energy"],
	    "instrumentalness": data["instrumentalness"],
	    "key": data["key"],
	    "liveness": data["liveness"],
	    "loudness": data["loudness"],
	    "mode": data["mode"],
	    "speechiness": data["speechiness"],
	    "tempo": data["tempo"],
	    "time_signature": data["time_signature"],
	    "track_href": data["track_href"],
	    "valence": data["valence"],
		}, index=[0])

		tracks_dataframe.append(original_track_frame)

	for track in object_track_recommendations_list:

		data = track.__dict__

		recommended_track_frame = pd.DataFrame({
		"seed": data["seed"],
		"id": data["id"],
		"added_at": data["added_at"],
		"added_by": data["added_by"],
		"artist_name": data["artist_name"],
		"release_date": data["release_date"],
		"album_name": data["album_name"],		
		"popularity": data["popularity"],
		"track_number": data["track_number"],		
		"track_name": data["track_name"],
		"disc_number": data["disc_number"],
		"album_id": data["album_id"],
		"album_genres": data["album_genres"],
		"artist_id": data["artist_id"],
		"acousticness": data["acousticness"],
	    "danceability": data["danceability"],
	    "duration_ms": data["duration_ms"],
	    "energy": data["energy"],
	    "instrumentalness": data["instrumentalness"],
	    "key": data["key"],
	    "liveness": data["liveness"],
	    "loudness": data["loudness"],
	    "mode": data["mode"],
	    "speechiness": data["speechiness"],
	    "tempo": data["tempo"],
	    "time_signature": data["time_signature"],
	    "track_href": data["track_href"],
	    "valence": data["valence"],
		}, index=[0])

		tracks_dataframe.append(recommended_track_frame)

	all_tracks_frame = pd.concat(tracks_dataframe, ignore_index=True)

	all_tracks_frame.to_excel(playlist+"_all_tracks_output.xlsx")

###
###
###   ARTISTS OUTPUT DATAFRAMES 
###
###

	artists_dataframe = []

	for artist in original_artists_list:

		data = artist.__dict__

		original_artists_frame = pd.DataFrame({
		"origin": data["seed"],
		"external_urls": data["external_urls"],
		"followers": data["followers"],
		"genres": data["genres"],
		"genres_1": data["genres_1"],
		"genres_2": data["genres_2"],
		"href": data["href"],
		"id": data["id"],
		"name": data["name"],
		"popularity": data["popularity"],
		"type": data["type"],
		"uri": data["uri"],
		}, index=[0])

		artists_dataframe.append(original_artists_frame)

	for artist in recommended_artists_list:

		data = artist.__dict__

		recommended_artists_frame = pd.DataFrame({
		"origin": data["seed"],
		"external_urls": data["external_urls"],
		"followers": data["followers"],
		"genres": data["genres"],
		"genres_1": data["genres_1"],
		"genres_2": data["genres_2"],
		"href": data["href"],
		"id": data["id"],
		"name": data["name"],
		"popularity": data["popularity"],
		"type": data["type"],
		"uri": data["uri"],
		}, index=[0])

		artists_dataframe.append(recommended_artists_frame)

	all_artists_frame = pd.concat(artists_dataframe, ignore_index=True)

	all_artists_frame.to_excel(playlist+"_all_artists_output.xlsx")




import pandas as pd

def graph_generator(object_track_recommendations_list, recommended_artists_list, playlist):

	track_nodes_pairs_list = []
	artist_nodes_pairs_list = []

	for track in object_track_recommendations_list:
		
		data = track.__dict__
		source = data["seed"]
		target = data["id"]

		node_pair = str(source)+";"+str(target)
		
		track_nodes_pairs_list.append(node_pair)

		print("Creating edges: " + node_pair)

	graph_frame = pd.DataFrame(track_nodes_pairs_list)
	graph_frame.to_csv(playlist+"_recommended_tracks_graph.csv", index=False, header=False)

	for artist in recommended_artists_list:
		
		data = artist.__dict__
		source = data["seed"]
		target = data["id"]

		node_pair = str(source)+";"+str(target)
		
		artist_nodes_pairs_list.append(node_pair)

		print("Creating edges: " + node_pair)

	graph_frame = pd.DataFrame(artist_nodes_pairs_list)
	graph_frame.to_csv(playlist+"_recommended_artists_graph.csv", index=False, header=False)


import spotipy
from spotipy.oauth2 import SpotifyClientCredentials
import spotixplore.credentials as cr
import spotixplore.classes.artist_class as ar


### CREDENTIALS

api_client_id = cr.SPOTIPY_CLIENT_ID
api_client_secret = cr.SPOTIPY_CLIENT_SECRET

sp = spotipy.Spotify(client_credentials_manager=SpotifyClientCredentials(api_client_id, api_client_secret))


def get_artists(object_track_list):

	original_artists_list = []
	recommended_artists_list = []
	artists_seeds = []

	for track in object_track_list:
		data = track.__dict__
		artist_id = data["artist_id"]
		artists_seeds.append(artist_id)

		print("Getting artist info for: "+artist_id)

		a_info = sp.artist(artist_id)
		try:
			artist_genres = a_info["genres"][0]
		except IndexError:
			artist_genres = "Nan"
		try:
			genres_1 = a_info["genres"][1]
		except IndexError:
			genres_1 = "Nan"
		try:
			genres_2 = a_info["genres"][2]
		except IndexError:
			genres_2 = "Nan"

		artist_object = ar.Artist(
		"from playlist",
		a_info["external_urls"].get("spotify", "Nan"),
		a_info["followers"].get("total", "Nan"),
		artist_genres,
		genres_1,
		genres_2,
		a_info.get("href", "Nan"),
		a_info.get("id", "Nan"),
		a_info.get("name", "Nan"),
		a_info.get("popularity", "Nan"),
		a_info.get("type", "Nan"),
		a_info.get("uri", "Nan"),
		)
		original_artists_list.append(artist_object)

	for seed in artists_seeds:

		print("Getting related artitsts for: "+seed)
		a_info = sp.artist_related_artists(seed)
		artists = a_info["artists"]

		for artist in artists:
			try:
				artist_genres = artist["genres"][0]
			except IndexError:
				artist_genres = "Nan"
			try:
				genres_1 = artist["genres"][1]
			except IndexError:
				genres_1 = "Nan"
			try:
				genres_2 = artist["genres"][2]
			except IndexError:
				genres_2 = "Nan"

			artist_object = ar.Artist(
				seed,
				artist["external_urls"].get("spotify", "Nan"),
				artist["followers"].get("total", "Nan"),
				artist_genres,
				genres_1,
				genres_2,
				artist.get("href", "Nan"),
				artist.get("id", "Nan"),
				artist.get("name", "Nan"),
				artist.get("popularity", "Nan"),
				artist.get("type", "Nan"),
				artist.get("uri", "Nan"),
				)

			recommended_artists_list.append(artist_object)

	return original_artists_list, recommended_artists_list


import spotixplore.starting_point as st

import spotixplore.extraction.get_tracks as gt
import spotixplore.extraction.get_recommended_tracks as gr
import spotixplore.extraction.get_artists as ga

import spotixplore.output.graph as tg
import spotixplore.output.dataframing as df

# First, you need to define credentials in "credentials.py"
# Second, you have to define playlists in "starting_pint.py"

playlists = st.PLAYLISTS

def spotixplore(playlists):
	for playlist in playlists:
			object_track_list, seeds_list = gt.get_tracks_from_playlist(playlist)
			object_track_recommendations_list, recom_tracks_id_list = gr.get_recommended_from_seeds(seeds_list)

			original_artists_list, recommended_artists_list = ga.get_artists(object_track_list)

			tg.graph_generator(object_track_recommendations_list, recommended_artists_list, playlist)
			df.dataframing(object_track_list, object_track_recommendations_list, original_artists_list, recommended_artists_list, playlist)

			print("Job Done!")


if __name__ == "__main__":
	spotixplore(playlists)
	









class Artist():
	
	def __init__(self,
		seed,
		external_urls,
		followers,
		genres,
		genres_1,
		genres_2,
		href,
		id,
		name,
		popularity,
		type,
		uri,
		):
		self.seed = seed
		self.external_urls = external_urls
		self.followers = followers
		self.genres = genres
		self.genres_1 = genres_1
		self.genres_2 = genres_2
		self.href = href
		self.id = id
		self.name = name
		self.popularity = popularity
		self.type = type
		self.uri = uri


import spotipy
from spotipy.oauth2 import SpotifyClientCredentials
import spotixplore.credentials as cr
import spotixplore.classes.track_class as tc


### CREDENTIALS

api_client_id = cr.SPOTIPY_CLIENT_ID
api_client_secret = cr.SPOTIPY_CLIENT_SECRET

sp = spotipy.Spotify(client_credentials_manager=SpotifyClientCredentials(api_client_id, api_client_secret))

def get_recommended_from_seeds(seeds_list):

	recom_tracks_objects_list = []
	recom_tracks_id_list = []

	# Open Playlist_data
	for seed in seeds_list:

		transitional_seed_list = []
		transitional_seed_list.append(seed)

		recommended_api_response = sp.recommendations(seed_tracks=transitional_seed_list, limit=5, country=None)

		#AFEGIT NOVA CONSULTA A LA API DE SPOTIFY PER RECOLLIR LES FEATURES DE CADA TRACK

		recommended_songs = recommended_api_response["tracks"]

		for track in recommended_songs:

			recommended_track_features = sp.audio_features(track["id"])
			tf = recommended_track_features[0]

			print("get audio features for recommended: "+tf["id"])
			recom_tracks_id_list.append(tf["id"])

			# GET ALBUM GENRES
			artist_id = track["artists"][0]["id"] 
			artist_info = sp.artist(artist_id)
			try:
				main_genre = artist_info["genres"][0]
			except IndexError:
				main_genre = "not_classified"

			try:
				info = tc.Track(
					seed,
					track.get("id", "Nan"),
					"recommended",
					"recommended",
					track["artists"][0].get("name", "Nan"),
					track["album"].get("release_date", "Nan"),
					track["album"].get("name", "Nan"),
					track.get("popularity", "Nan"),
					track.get("track_number", "Nan"),
					track.get("name", "Nan"),
					track.get("disc_number", "Nan"),
					track["album"].get("id", "Nan"),
					main_genre,
					artist_id,
					tf.get("acousticness", "Nan"),
					tf.get("analysis_url", "Nan"),
					tf.get("danceability", "Nan"),
					tf.get("duration_ms", "Nan"),
					tf.get("energy", "Nan"),
					tf.get("instrumentalness", "Nan"),
					tf.get("key", "Nan"),
					tf.get("liveness", "Nan"),
					tf.get("loudness", "Nan"),
					tf.get("mode", "Nan"),
					tf.get("speechiness", "Nan"),
					tf.get("tempo", "Nan"),
					tf.get("time_signature", "Nan"),
					tf.get("track_href", "Nan"),
					tf.get("type", "Nan"),
					tf.get("uri", "Nan"),
					tf.get("valence", "Nan")
					)
			except AttributeError:
				continue

			recom_tracks_objects_list.append(info)

	return(recom_tracks_objects_list, recom_tracks_id_list)